<?xml version="1.0" encoding="UTF-8"?>
<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<gui name="Granatier" version="1">
    <MenuBar>
        <Menu name="game">
            <Action name="level" />
        </Menu>
        <Menu name="settings">
            <Action name="sounds" />
        </Menu>
    </MenuBar>
</gui>

